@@ -1,4 +1,5 @@
 """For entities that have a condition template."""
+from gemd.entity.link_by_uid import LinkByUID
 from gemd.entity.setters import validate_list
 from gemd.entity.template.base_template import BaseTemplate
 from gemd.entity.template.condition_template import ConditionTemplate
@@ -34,7 +35,7 @@ class HasConditionTemplates(object):
 
     @conditions.setter
     def conditions(self, conditions):
-        lst = validate_list(conditions, (ConditionTemplate, list, tuple))
+        lst = validate_list(conditions, (ConditionTemplate, LinkByUID, list, tuple))
 
         # make sure attribute can be a Condition
         # TODO: list.map(_.validate_scope(AttributeType.CONDITION)) all true
